@@ -1,11 +1,12 @@
 import cv2
-import numpy as np
 import os
 import glob
 import multiprocessing as mp
 import argparse
-import processingtools.PrgressBar
+import processingtools.ProgressBar
 import time
+import warnings
+import typing
 
 
 class VideoTools:
@@ -38,7 +39,7 @@ class VideoTools:
         self.height = round(self.cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
         self.video_name = os.path.basename(self.video_path)[:-4]
 
-    def video2images(self, save_path: str, extension: str = 'jpg', start: float = 0, end: float = None, jump: float = 1, option: str = 'frame', size=None):
+    def video2images(self, save_path: str, extension: str = 'jpg', start: float = 0, end: float = None, jump: float = 1, option: str = 'frame', size=None) -> True:
         """
         video to image files
         :param save_path: video file directory, save_path: save png directory
@@ -64,7 +65,7 @@ class VideoTools:
 
         create_folder(save_path)
 
-        for n, i in processingtools.PrgressBar.ProgressBar(enumerate(range(self.length)), total=self.length, finish_mark=None):
+        for n, i in processingtools.ProgressBar(enumerate(range(self.length)), total=self.length, finish_mark=None):
             ret, frame = self.cap.read()
             if start <= n and ret and n % jump == 0:
                 frame = frame if size is None else cv2.resize(frame, (size[1], size[0]))
@@ -78,7 +79,7 @@ class VideoTools:
 
         return True
 
-    def video_resize(self, save_path: str, size):
+    def video_resize(self, save_path: str, size) -> True:
         """
         video resize as size
         :param save_path: save_path path
@@ -92,7 +93,7 @@ class VideoTools:
             size = [round(self.height * size), round(self.width * size)]
         out = cv2.VideoWriter(save_path, self.fourcc, self.fps, (size[1], size[0]))
 
-        for _ in processingtools.PrgressBar.ProgressBar(range(self.length), total=self.length, finish_mark=None):
+        for _ in processingtools.ProgressBar(range(self.length), total=self.length, finish_mark=None):
             _, frame = self.cap.read()
             out.write(cv2.resize(frame, (size[1], size[0])))
 
@@ -129,72 +130,54 @@ class MultiProcess:
 
         self.cpu_n = cpu_n
 
-    def duplicate_func(self, func, args: tuple):
+    def duplicate_func(self, func, args_list: typing.Union[tuple, list], progress_args: typing.Union[dict, bool] = True):
         """
         Run the function as multiprocess
         :param func: the function for running multiprocess
-        :param args: arguments for function
+        :param args_list: arguments for function
+        :param progress_args: arguments for ProgressBar. if False, it doesn't use progressbar, if True it uses progressbar.
         :return: True
         """
 
-        i = 0
-        j = 0
-
-        if self.cpu_n < len(args):
-            for i in range(len(args) // self.cpu_n):
-                pro = list()
-                for j in range(self.cpu_n):
-                    pro.append(mp.Process(target=func, args=args[i * self.cpu_n + j]))
-                for mul in pro: mul.start()
-                for mul in pro: mul.join()
-
-            pro = list()
-            for left in range(self.cpu_n * i + j + 1, len(args)):
-                pro.append(mp.Process(target=func, args=args[left]))
-            for mul in pro: mul.start()
-            for mul in pro: mul.join()
-
-        else:
-            pro = list()
-            for left in range(0, len(args)):
-                pro.append(mp.Process(target=func, args=args[left]))
-            for mul in pro: mul.start()
-            for mul in pro: mul.join()
+        if type(progress_args) is not dict and type(progress_args) is not bool:
+            raise TypeError('type of progress_args is needed dict or bool')
 
+        with mp.Pool(processes=self.cpu_n) as pool:
+            results = [pool.apply_async(func, args) for args in args_list]
+            if progress_args is True:
+                for result in processingtools.ProgressBar(results):
+                    result.get()
+            elif progress_args:
+                for result in processingtools.ProgressBar(results, **progress_args):
+                    result.get()
+            else:
+                for result in results:
+                    result.get()
         return True
 
-    def multi_func(self, funcs: tuple, args: tuple):
+    def multi_func(self, funcs: typing.Union[tuple, list], args: typing.Union[tuple, list], progress_args: typing.Union[dict, bool] = True):
         """
         Run the function as multiprocess
         :param funcs: the functions for running multiprocess
         :param args: arguments for function
+        :param progress_args: arguments for ProgressBar. if False, it doesn't use progressbar, if True it uses progressbar.
         :return: True
         """
 
-        i = 0
-        j = 0
-
-        if self.cpu_n < len(args):
-            for i in range(len(args) // self.cpu_n):
-                pro = list()
-                for j in range(self.cpu_n):
-                    pro.append(mp.Process(target=funcs[i * self.cpu_n + j], args=args[i * self.cpu_n + j]))
-                for mul in pro: mul.start()
-                for mul in pro: mul.join()
-
-            pro = list()
-            for left in range(self.cpu_n * i + j + 1, len(args)):
-                pro.append(mp.Process(target=funcs[left], args=args[left]))
-            for mul in pro: mul.start()
-            for mul in pro: mul.join()
-
-        else:
-            pro = list()
-            for left in range(0, len(args)):
-                pro.append(mp.Process(target=funcs[left], args=args[left]))
-            for mul in pro: mul.start()
-            for mul in pro: mul.join()
+        if type(progress_args) is not dict and type(progress_args) is not bool:
+            raise TypeError('type of progress_args is needed dict or bool')
 
+        with mp.Pool(processes=self.cpu_n) as pool:
+            results = [pool.apply_async(func, args) for func, args in zip(funcs, args)]
+            if progress_args is True:
+                for result in processingtools.ProgressBar(results):
+                    result.get()
+            elif progress_args:
+                for result in processingtools.ProgressBar(results, **progress_args):
+                    result.get()
+            else:
+                for result in results:
+                    result.get()
         return True
 
     def split_list(self, *args):
@@ -223,7 +206,7 @@ class MultiProcess:
         try:
             import dill
         except ModuleNotFoundError:
-            raise ModuleNotFoundError('MultiProcess is needed dill! Try <pip install dill>.')
+            raise ModuleNotFoundError('wrapper is needed dill! Try <pip install dill>.')
 
         func = dill.loads(data)
         return func(*args, **kwargs)
@@ -248,7 +231,7 @@ class MultiProcess:
         try:
             import dill
         except ModuleNotFoundError:
-            raise ModuleNotFoundError('MultiProcess is needed moviepy! Try <pip install dill>.')
+            raise ModuleNotFoundError('adapt_function is needed moviepy! Try <pip install dill>.')
 
         if order:
             def adapted_function(num, outputs: mp.queues.Queue, *args, **kwargs):
@@ -262,22 +245,74 @@ class MultiProcess:
         return dill.dumps(adapted_function)
 
 
-def create_folder(directory, warning: bool = True):
+class DeprecationWarningC(UserWarning):
+    """ Base class for warnings about deprecated features. """
+
+    def __init__(self, *args, **kwargs):  # real signature unknown
+        super().__init__(*args, **kwargs)
+
+
+def warning_format(message, category, filename, lineno, line=None):
+    """
+    Custom warning format for warnings module.
+    :param message: The warning message.
+    :param category: The category of the warning.
+    :param filename: The name of the file in which the warning was raised.
+    :param lineno: The line number where the warning was raised.
+    :param line: The line of code that raised the warning.
+    :return: A string containing a warning message.
+    """
+
+    return f'{category.__name__}: {message}\n'
+
+
+def custom_warning_format(func):
+    """
+    Decorator for applying a custom warning format to a function.
+    :param func: The function to apply the custom warning format to.
+    :return: The decorated function.
+    """
+
+    def wrapper(*args, **kwargs):
+        original_format = warnings.formatwarning
+        warnings.formatwarning = warning_format
+        try:
+            result = func(*args, **kwargs)
+        finally:
+            warnings.formatwarning = original_format
+        return result
+    return wrapper
+
+
+@custom_warning_format
+def create_folder(directory, print_warning: bool = True, warning=None):
     """
     create folder when folder is not exist
     :param directory: the path which is verified exist
-    :param warning: print warning when folder is not exist
+    :param print_warning: print warning when folder is not exist
+    :param warning: print warning when folder is not exist (argument warning will be deprecated in the next version. Use the print_warning instead.)
     :return: True when directory is created
     """
 
+    if warning is not None:
+        warnings.warn(f'argument warning will be deprecated in the next version. Use the print_warning instead.', DeprecationWarning)
+        warnings.warn(f'argument warning will be deprecated in the next version. Use the print_warning instead.', DeprecationWarningC)
+        print_warning = warning
+
+    directory = os.path.abspath(directory)
+
     try:
         if not os.path.exists(directory):
-            print(f'\033[31m{directory} is created. \033[0m') if warning else None
+            if print_warning:
+                print(f'{directory} is created.')
             os.makedirs(directory)
             return True
-        else: return False
+        else:
+            if print_warning:
+                warnings.warn(f'{directory} is already exist.')
+            return False
     except OSError:
-        print('Error: Creating directory. ' + directory)
+        raise OSError(f'Error: Cannot create directory. ({directory})')
 
 
 def read_images_list(dir_path: str, img_format: str = None):
@@ -314,7 +349,8 @@ def read_images(dir_path: str, img_format: str = None):
         return [cv2.imread(_) for _ in sorted(glob.glob(f'{dir_path}/*.{img_format}'))]
 
 
-def multi_func(func, args: tuple, cpu_n: int = mp.cpu_count()):
+@custom_warning_format
+def multi_func(func, args: tuple, cpu_n: int = mp.cpu_count()) -> True:
     """
     Run the function as multiprocess
     :param func: the function for running multiprocess
@@ -323,6 +359,9 @@ def multi_func(func, args: tuple, cpu_n: int = mp.cpu_count()):
     :return: True
     """
 
+    warnings.warn(f'{multi_func.__name__} will be deprecated in the next version. Use the class MultiProcess instead.', DeprecationWarning)
+    warnings.warn(f'{multi_func.__name__} will be deprecated in the next version. Use the class MultiProcess instead.', DeprecationWarningC)
+
     i = 0
     j = 0
 
@@ -350,16 +389,20 @@ def multi_func(func, args: tuple, cpu_n: int = mp.cpu_count()):
     return True
 
 
+@custom_warning_format
 def png2video(images_path: str, save_path: str, fps: int = 60, fourcc: int = cv2.VideoWriter_fourcc(*'DIVX')):
     """
     make avi file using images in path
     :param images_path: directory path for images
     :param save_path: directory path for video
-    :param fps: video fps (default: 60)
+    :param fps:  frame per second (default: 60)
     :param fourcc: video fourcc (default: cv2.VideoWriter_fourcc(*'DIVX'))
     :return: True
     """
 
+    warnings.warn(f'{png2video.__name__} will be deprecated in the next version. Use the class VideoTools instead.', DeprecationWarning)
+    warnings.warn(f'{png2video.__name__} will be deprecated in the next version. Use the class VideoTools instead.', DeprecationWarningC)
+
     # when run in window, should replace backslash
     images_path = images_path.replace('\\', '/')
     save_path = save_path.replace('\\', '/')
@@ -373,7 +416,7 @@ def png2video(images_path: str, save_path: str, fps: int = 60, fourcc: int = cv2
     h, w, _ = cv2.imread(files[0]).shape
     out = cv2.VideoWriter(save_path, fourcc, fps, (w, h))
 
-    for i in processingtools.PrgressBar.ProgressBar(files):
+    for i in processingtools.ProgressBar(files):
         out.write(cv2.imread(i))
 
     out.release()
@@ -481,18 +524,63 @@ def timer(input_function):
 
 
 def zero_padding(max_num, present_num):
-    n_zero = int(np.log10(max_num)) + 1
-    zeros = f'0{n_zero}d'
-
-    return f'{present_num:{zeros}}'
-
-
-def print_style(text, foreground_rgb=None, background_rgb=None, styles: tuple = (), sep=' ', end='\n', file=None) -> None:
     """
-    Prints the given text with specified color and style.
+    this function is used to pad zeros to a number based on the maximum number in the dataset.
+    :param max_num: the maximum number in the dataset. it is used to determine the total number of digits.
+    :param present_num: the current number that needs to be padded with zeros.
+    :return: the zero-padded result.
+    """
+
+    num_digits = len(str(max_num))
+    format_string = f'0{num_digits}d'
+
+    return f'{present_num:{format_string}}'
+
+
+def s_text(text, f_rgb=None, b_rgb=None, styles: tuple = ()) -> str:
+    """
+    prints the given text with specified color and style.
     :param text: the text to be printed
-    :param foreground_rgb: the RGB color code for the text color
-    :param background_rgb: the RGB color code for the background color
+    :param f_rgb: the RGB color code for the text color
+    :param b_rgb: the RGB color code for the background color
+    :param styles: the styles to be applied to the text. Options are 'bold', 'tilt', 'underscore', and 'cancel'
+    :return: str
+    """
+
+    # define style codes
+    style_codes = {
+        'bold': '\033[1m',
+        'tilt': '\033[3m',
+        'underscore': '\033[4m',
+        'cancel': '\033[9m',
+        'flicker': '\033[5m'
+    }
+
+    # apply styles to the text
+    for style in styles:
+        if style not in style_codes:
+            raise ValueError(f"Invalid styles: {style}. Valid options are: {list(style_codes.keys())}")
+        text = f'{style_codes.get(style, "")}{text}'
+
+    # set text color
+    if f_rgb:
+        foreground_rgb = [max(0, min(255, int(c))) for c in f_rgb[:3]]
+        text = f'\033[38;2;{foreground_rgb[0]};{foreground_rgb[1]};{foreground_rgb[2]}m{text}'
+
+    # set background color
+    if b_rgb:
+        background_rgb = [max(0, min(255, int(c))) for c in b_rgb[:3]]
+        text = f'\033[48;2;{background_rgb[0]};{background_rgb[1]};{background_rgb[2]}m{text}'
+
+    return text
+
+
+def sprint(text, f_rgb=None, b_rgb=None, styles: tuple = (), sep=' ', end='\n', file=None) -> None:
+    """
+    prints the given text with specified color and style.
+    :param text: the text to be printed
+    :param f_rgb: the RGB color code for the text color
+    :param b_rgb: the RGB color code for the background color
     :param styles: the styles to be applied to the text. Options are 'bold', 'tilt', 'underscore', and 'cancel'
     :param sep: the separator to be used in the print function
     :param end: the end character to be used in the print function
@@ -500,39 +588,4 @@ def print_style(text, foreground_rgb=None, background_rgb=None, styles: tuple =
     :return: None
     """
 
-    if background_rgb is None:
-        background_rgb = []
-    if foreground_rgb is None:
-        foreground_rgb = []
-
-    if 'bold' in styles:
-        text = f'\033[1m{text}'
-    if 'tilt' in styles:
-        text = f'\033[3m{text}'
-    if 'underscore' in styles:
-        text = f'\033[4m{text}'
-    if 'cancel' in styles:
-        text = f'\033[9m{text}'
-
-    i = 0
-    while i < 3 and i < len(foreground_rgb):
-        foreground_rgb[i] = int(foreground_rgb[i])
-        if foreground_rgb[i] < 0:
-            foreground_rgb[i] = 0
-        elif foreground_rgb[i] > 255:
-            foreground_rgb[i] = 255
-        i += 1
-
-    i = 0
-    while i < 3 and i < len(background_rgb):
-        background_rgb[i] = int(background_rgb[i])
-        if background_rgb[i] < 0:
-            background_rgb[i] = 0
-        elif background_rgb[i] > 255:
-            background_rgb[i] = 255
-        i += 1
-
-    text = f'\033[38;2;{foreground_rgb[0]};{foreground_rgb[1]};{foreground_rgb[2]}m{text}' if len(foreground_rgb) == 3 else text
-    text = f'\033[48;2;{background_rgb[0]};{background_rgb[1]};{background_rgb[2]}m{text}' if len(background_rgb) == 3 else text
-
-    print(f'{text}\033[0m', sep=sep, end=end, file=file)
+    print(f'{s_text(text, f_rgb, b_rgb, styles)}\033[0m', sep=sep, end=end, file=file)
