@@ -6,6 +6,9 @@ import torch
 import torchvision.transforms
 import torchvision.transforms.functional
 import torch.nn.functional
+import typing
+import warnings
+import processingtools.functions
 
 
 class QueueTensor:
@@ -148,6 +151,53 @@ class GradCam:
         return True
 
 
+class HOA(torch.nn.Module):
+    """
+    High Order Attention module.
+    Chen, Binghui, Weihong Deng, and Jiani Hu. "Mixed high-order attention network for person re-identification."
+        Proceedings of the IEEE/CVF international conference on computer vision. 2019.
+    https://openaccess.thecvf.com/content_ICCV_2019/papers/Chen_Mixed_High-Order_Attention_Network_for_Person_Re-Identification_ICCV_2019_paper.pdf
+    """
+
+    def __init__(self, in_channels: int, r: int = 3, activation: str = 'ReLU') -> None:
+        """
+        initial function
+        :param in_channels: the number of input channels
+        :param r: order for attention module
+        :param activation: activation function for MOA
+        """
+
+        super().__init__()
+
+        conv2ds = []
+        for i in range(r):
+            conv2ds.append(torch.nn.ModuleList([torch.nn.Conv2d(in_channels, in_channels, 1) for _ in range(i + 1)]))
+        self.conv2ds = torch.nn.ModuleList(conv2ds)
+
+        sequentials = [torch.nn.Sequential(getattr(torch.nn, activation)(), torch.nn.Conv2d(in_channels, in_channels, 1)) for _ in range(r)]
+        self.sequentials = torch.nn.ModuleList(sequentials)
+
+        self.sigmoid = torch.nn.Sigmoid()
+
+    def forward(self, x) -> torch.Tensor:
+        cs = []
+        for conv2d, sequential in zip(self.conv2ds, self.sequentials):
+            z1s = [_(x) for _ in conv2d]
+
+            c = torch.ones_like(x)
+            for z1 in z1s:
+                c *= z1
+
+            c = sequential(c)
+            cs.append(c)
+
+        outs = torch.zeros_like(x)
+        for c in cs:
+            outs += c
+
+        return outs * x
+
+
 def torch_img_show(img):
     """
     :param img: torch tensor image
@@ -474,3 +524,104 @@ class SecondOrderDerivative(torch.nn.Module):
             if x is not None:
                 flatt_grad.append(torch.reshape(x, (-1,)))
         return self.epsilon / torch.norm(torch.cat(flatt_grad)), grad_val
+
+
+class AutoInputModel(torch.nn.Module):
+    """
+    A PyTorch module for automatically processing and normalizing input images.
+
+    This class wraps a given model and provides functionality to read, preprocess, and forward images through the model.
+    It supports custom transformers and normalization parameters.
+    """
+
+    @processingtools.functions.custom_warning_format
+    def __init__(self, model, size: typing.Union[tuple, list], mean: typing.Union[float, list, torch.Tensor, None] = None, std: typing.Union[float, list, torch.Tensor, None] = None, transformer=None):
+        """
+        initialize the AutoInputModel
+        :param model: model to be used
+        :param size: size to which images will be resized
+        :param mean: mean for normalization
+        :param std: standard deviation for normalization
+        :param transformer: custom transformer for image preprocessing
+        """
+
+        if transformer is None and (mean is None or std is None):
+            raise ValueError('Either transformer must be provided, or both mean and std must be specified.')
+
+        super().__init__()
+
+        self.model = model
+
+        if transformer is not None and (mean is not None or std is not None):
+            warnings.warn('NormalizeModel uses transformer for normalizing not (mean, std).')
+
+        if transformer is not None:
+            self.transform = transformer
+        else:
+            self.transform = torchvision.transforms.Compose([
+                torchvision.transforms.Resize(size=size),
+                torchvision.transforms.ToTensor(),
+                torchvision.transforms.Normalize(mean, std),
+            ])
+
+    def image_read(self, path: str) -> torch.Tensor:
+        """
+        read and preprocess an image from the given path
+        :param path: image file path
+        :return: normalized image tensor
+        """
+
+        try:
+            image = torch.from_numpy(cv2.cvtColor(cv2.imread(path), cv2.COLOR_BGR2RGB)).float().permute(2, 0, 1)[None]
+            return self.transform(image)
+        except Exception as e:
+            raise ValueError(f'Error reading image from path {path}: {e}')
+
+    def get_device(self, device=None) -> None:
+        """
+        get the device to run the model on
+        :param device: device to run the model on
+        :return: device
+        """
+
+        device = next(self.model.parameters()).device if device is None else device
+        print(f'run on {processingtools.functions.s_text(f"{device}", styles=("bold",))}')
+
+    @processingtools.functions.custom_warning_format
+    def forward(self, inputs: typing.Union[str, list], device=None) -> torch.Tensor:
+        """
+        forward pass of the model
+        :param inputs: string or list of strings representing image paths
+        :param device: device to run the model on
+        :return: model outputs
+        """
+
+        if self.model.training:
+            warnings.warn('model is training mode now! (if you want to change eval mode, add model.eval())')
+
+        self.get_device(device)
+
+        outputs = []
+        if isinstance(inputs, list):
+            for i in inputs:
+                outputs.append(self.model(self.image_read(i)))
+            return torch.cat(outputs, dim=0)
+        elif isinstance(inputs, str):
+            return self.model(self.image_read(inputs))
+        else:
+            raise TypeError('inputs must be a string or a list of strings')
+
+
+def gpu_env() -> None:
+    """
+    check and display GPU availability and details.
+    :return: None
+    """
+
+    num_gpus = torch.cuda.device_count()
+
+    print(f'GPU usable: {"Yes" if torch.cuda.is_available() else "No"}')
+    print(f'Number of detected GPUs: {num_gpus}')
+
+    for i in range(num_gpus):
+        print(f'    • GPU {i}: {torch.cuda.get_device_name(i)}')
